import { matchPath } from "react-router-dom";
import spec_data from "./fixtures/api/example-spec.json";
import hosts_data from "./fixtures/api/hosts.json";

// Utility to extract search params from the
// String representation of the request url
// (Swagger client's request is a fetch Request,
// so the url is a String and not a URL object)
function searchParams({ request }) {
  if (typeof request.url === "string") {
    const url = new URL(request.url);
    return url.searchParams;
  } else {
    return request.url.searchParams;
  }
}

function getParameters(req, pattern) {
  const match = matchPath({ path: pattern }, new URL(req.request.url).pathname);
  return match?.params;
}

export function withMockPagination({ page, limit, data, key }) {
  const startIndex = page * limit;
  const resultList = key ? data[key] : data;
  const pagedData = resultList.slice(startIndex, startIndex + limit);

  const dataKey = key ?? "data";

  const results = {
    limit,
    listSize: pagedData.length,
    pageNumber: page,
    totalCount: resultList.length,
    [dataKey]: pagedData,
    DEVELOPER_NOTE: "THIS IS MOCKED PAGING!"
  };

  return results;
}

function requestWithMockPagination({ req, data, key }) {
  const params = searchParams(req);
  const page = Number(params.get("page")) || 0;
  const limit = Number(params.get("limit")) || 99999;
  return withMockPagination({ page, limit, data, key });
}

// handler to return the OpenAPI spec
function spec() {
  return {
    body: spec_data
  };
}

function get_host(req) {
  const params = getParameters(req, "/hosts/:id");
  const body = hosts_data.find((x) => x.id === Number(params.id));
  if (body) {
    return { body };
  } else {
    return { status: 404 };
  }
}

function list_hosts(req) {
  const {
    // page = 0,
    // limit = 10,
    query = ""
  } = Object.fromEntries(searchParams(req));
  const filteredData = query
    ? hosts_data.filter((h) => h.name.includes(query))
    : hosts_data;

  const { data, ...rest } = requestWithMockPagination({
    req,
    data: filteredData
  });
  return {
    body: {
      ...rest,
      hostList: data
    }
  };
}

function basic() {
  const message = "hi";
  return {
    body: { message }
  };
}

async function echo(req) {
  const { queryEcho } = Object.fromEntries(searchParams(req));
  const body = await req.request.json();
  const { bodyEcho } = body;
  return {
    body: { queryEcho, bodyEcho }
  };
}

async function api_login(req) {
  const body = await req.request.json();
  if (body.username?.length > 0 && body.password?.length > 0) {
    return {
      body: { token: req.request.body.username }
    };
  }

  return { body: { message: "Bad user name or password" }, status: 401 };
}

// just to organize
const mockAPI = {
  spec: spec,
  hosts: {
    list_hosts,
    get_host
  },
  echo: {
    echo
  },
  basic: { basic },
  auth: {
    api_login
  }
};

// Enough info to create handlers in cypress or MSW
const makeHandler = (method, matcher, requestHandler, wrapper, delay) => ({
  method,
  matcher,
  requestHandler: wrapper ? wrapper(requestHandler) : requestHandler,
  name: requestHandler.name, // use caution when changing api function names
  delay: delay
});

const queryPattern = "(\\?.*)?$";
const qRegExp = (pattern) => RegExp(pattern + queryPattern);

// Handlers for our whole API
export const apiHandlers = [
  // api spec
  makeHandler("GET", qRegExp(".*/spec"), mockAPI.spec),

  // hosts
  makeHandler(
    "GET",
    qRegExp(".*/hosts/[0-9]+"),
    mockAPI.hosts.get_host,
    null,
    2000
  ),
  makeHandler("GET", qRegExp(".*/hosts"), mockAPI.hosts.list_hosts),

  // echo
  makeHandler("POST", qRegExp(".*/echo"), mockAPI.echo.echo),

  // basic_return
  makeHandler("GET", qRegExp(".*/basic"), mockAPI.basic.basic),

  // login
  makeHandler("POST", qRegExp(".*/api_login"), mockAPI.auth.api_login)
];
